<?php
/***
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\ExtViSualMerchandiser\Block\Adminhtml\Category\Merchandiser;

use FiloBlu\ExtViSualMerchandiser\Model\Category\Products;
use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Grid;
use Magento\Backend\Helper\Data;
use Magento\Catalog\Helper\Image;
use Magento\Catalog\Model\Product;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Collection;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Registry;
use Magento\Framework\View\Element\AbstractBlock;
use Magento\VisualMerchandiser\Block\Adminhtml\Widget\Tile\Attribute\Factory;

/**
 * Class Tile
 * @package FiloBlu\ExtViSualMerchandiser\Block\Adminhtml\Category\Merchandiser
 */
class Tile extends Grid
{
    /** @var string */
    const XML_PATH_ADDITIONAL_ATTRIBUTES = 'visualmerchandiser/options/product_attributes';
    /** @var string */
    const XML_PATH_IMAGE_WIDTH = 'visualmerchandiser/images/width';
    /** @var string */
    const XML_PATH_IMAGE_HEIGHT = 'visualmerchandiser/images/height';
    /** @var string */
    const XML_PATH_IMAGE_TYPE = 'visualmerchandiser/images/type';
    /** @var int */
    const IMAGE_WIDTH = 130;
    /** @var int */
    const IMAGE_HEIGHT = 130;
    /** @var ScopeConfigInterface */
    public $scopeConfig;
    /** @var string */
    protected $_template = 'category/merchandiser/tile.phtml';

    /**
     * Core registry
     *
     * @var Registry
     */
    protected $_coreRegistry;
    /**
     * Collection object
     *
     * @var Collection
     */
    protected $_collection;

    /**
     * Catalog image
     *
     * @var Image
     */
    protected $_catalogImage;

    /** @var \Magento\VisualMerchandiser\Model\Category\Products */
    protected $_products;

    /** @var array */
    protected $usableAttributes;

    /** @var ObjectManagerInterface */
    protected $attributeFactory;

    /**
     * @param Context $context
     * @param Data $backendHelper
     * @param Registry $coreRegistry
     * @param Image $catalogImage
     * @param Products $products
     * @param Factory $attributeFactory
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $backendHelper,
        Registry $coreRegistry,
        Image $catalogImage,
        Products $products,
        Factory $attributeFactory,
        array $data = []
    ) {
        parent::__construct($context, $backendHelper, $data);
        $this->_coreRegistry = $coreRegistry;
        $this->_catalogImage = $catalogImage;
        $this->_products = $products;
        $this->scopeConfig = $context->getScopeConfig();
        $this->attributeFactory = $attributeFactory;
    }

    /**
     * @param Product $product
     * @return string
     */
    public function getImageUrl($product)
    {
        $image = $this->getImageHelper()
            ->init(
                $product,
                $this->scopeConfig->getValue(self::XML_PATH_IMAGE_TYPE),
                [
                    'type' => $this->scopeConfig->getValue(self::XML_PATH_IMAGE_TYPE)
                ]
            )
            ->resize(
                $this->scopeConfig->getValue(self::XML_PATH_IMAGE_WIDTH),
                $this->scopeConfig->getValue(self::XML_PATH_IMAGE_HEIGHT)
            );
        return $image->getUrl();
    }

    /**
     * @return Image
     */
    public function getImageHelper()
    {
        return $this->_catalogImage;
    }

    /**
     * get collection object
     *
     * @return Collection
     */
    public function getCollection()
    {
        return $this->_collection;
    }

    /**
     * Set collection object
     *
     * @param Collection $collection
     * @return void
     */
    public function setCollection($collection)
    {
        $this->_collection = $collection;
    }

    /**
     * Retrieve column by id
     *
     * @param string $columnId
     * @return AbstractBlock|bool
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function getColumn($columnId)
    {
        return false;
    }

    /**
     * Retrieve list of grid columns
     *
     * @return array
     */
    public function getColumns()
    {
        return [];
    }

    /**
     * @return array|null
     */
    public function getCategory()
    {
        return $this->_coreRegistry->registry('category');
    }

    /**
     * @return string
     */
    public function getGridUrl()
    {
        return $this->getUrl('merchandiser/*/tile', ['_current' => true]);
    }

    /**
     * @param Product $product
     * @return array
     */
    public function getAttributesToDisplay($product)
    {
        $attributeCodes = $this->getUsableAttributes();
        $availableAttributes = $product->getTypeInstance()->getSetAttributes($product);
        $availableFields = array_keys($product->getData());
        $filteredAttributes = [];

        foreach ($attributeCodes as $code) {
            $renderer = $this->attributeFactory->create($code);

            if ($code === 'price') {
                $attributeObject = $availableAttributes[$code];
                $filteredAttributes[] = $renderer->addData([
                    'label' => $attributeObject->getFrontend()->getLabel(),
                    'value' => $product->getFormatedPrice()
                ]);
            } elseif (isset($availableAttributes[$code])) {
                $attributeObject = $availableAttributes[$code];
                $filteredAttributes[] = $renderer->addData([
                    'label' => $attributeObject->getFrontend()->getLabel(),
                    'value' => $product->getData($code)
                ]);
            } elseif (in_array($code, $availableFields, true)) {
                $filteredAttributes[] = $renderer->addData([
                    'label' => ucwords($code),
                    'value' => $product->getData($code)
                ]);
            }
        }

        return $filteredAttributes;
    }

    /**
     * @return array
     */
    protected function getUsableAttributes()
    {
        if ($this->usableAttributes === null) {
            $attributeCodes = (string)$this->scopeConfig->getValue(self::XML_PATH_ADDITIONAL_ATTRIBUTES);
            $attributeCodes = explode(',', $attributeCodes);
            $this->usableAttributes = array_map('trim', $attributeCodes);
        }
        return $this->usableAttributes;
    }

    /**
     * @return mixed
     */
    public function getImageWidth()
    {
        return $this->scopeConfig->getValue(self::XML_PATH_IMAGE_WIDTH);
    }

    /**
     * @return void
     */
    protected function _construct()
    {
        parent::_construct();
        $this->setDefaultSort('position');
        $this->setDefaultDir('asc');
        $this->setUseAjax(true);
    }

    /**
     * Initialize grid
     *
     * @return void
     * @throws LocalizedException
     */
    protected function _prepareGrid()
    {
        $this->_prepareCollection();
    }

    /**
     * @return $this
     * @throws LocalizedException
     */
    protected function _prepareCollection()
    {
        $this->_products->setCacheKey($this->getPositionCacheKey());

        $collection = $this->_products->getCollectionForGrid(
            (int)$this->getRequest()->getParam('id', 0)
        );

        $collection = $this->_products->applyCachedChanges($collection);

        $collection->clear();
        $this->setCollection($collection);

        $this->_preparePage();

        $idx = ($collection->getCurPage() * $collection->getPageSize()) - $collection->getPageSize();

        foreach ($collection as $item) {
            $item->setPosition($idx);
            $idx++;
        }

        return $this;
    }

    /**
     * @return string
     */
    protected function _getPositionCacheKey()
    {
        return $this->getPositionCacheKey() ?
            $this->getPositionCacheKey() :
            $this->getParentBlock()->getPositionCacheKey();
    }

    /**
     * Process column filtration values
     *
     * @param mixed $data
     * @return $this
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    protected function _setFilterValues($data)
    {
        return $this;
    }
}
